var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { useCallback, useRef } from 'react';
import { usePromise } from './usePromise';
import { useLatest } from '../generic/useLatest';
import { useReset } from '../state/useReset';
export function useAsync(fn) {
    const lastFn = useLatest(fn);
    // Unique call id to handle concurrency
    const callIdRef = useRef(0);
    // Inner state
    const [state, setState, reset] = useReset({
        isLoading: false,
        isCompleted: false,
        isSucceed: false,
        isFailed: false,
    });
    const runAsync = useCallback((...args) => __awaiter(this, void 0, void 0, function* () {
        const callId = ++callIdRef.current;
        setState({ isLoading: true, isCompleted: false, isSucceed: false, isFailed: false, args });
        try {
            // Execute
            const result = yield lastFn.current(...args);
            if (callIdRef.current === callId) {
                setState({ isLoading: false, isCompleted: true, isSucceed: true, isFailed: false, args, result });
            }
            return result;
        }
        catch (error) {
            if (callIdRef.current === callId) {
                setState({ isLoading: false, isCompleted: true, isSucceed: false, isFailed: true, args, error });
            }
            throw error;
        }
    }), []);
    const run = usePromise(runAsync);
    return Object.assign(Object.assign({}, state), { runAsync,
        run,
        reset });
}
