import { useCallback } from 'react';
import { useLatest } from './useLatest';
/**
 * Wraps useLatest and adds a useCallback. Produces a stable callback given
 * a possibly unstable input function. The returned function is granted to
 * be stable, but will always call the latest function passed to the hook
 */
export function useLatestCallback(fn) {
    const latest = useLatest(fn);
    return useCallback((...args) => latest.current(...args), []);
}
